See the [benchmarks](#benchmarks). gray-matter is 20-30x faster than [front-matter][].

## Highlights

* Reliable and battle-tested by [metalsmith](https://github.com/segmentio/metalsmith), [assemble](https://github.com/assemble/assemble), [verb](https://github.com/assemble/verb), and many other projects!
* Extracts and parses:
  - [YAML](http://github.com/nodeca/js-yaml)
  - [JSON](http://en.wikipedia.org/wiki/Json)
  - [TOML](http://github.com/mojombo/toml) (must be installed first)
  - [CoffeeScript](http://coffeescript.org) when `options.eval` is set to `true` (must be installed first)
  - [CSON](https://github.com/bevry/cson) when `options.eval` is set to `true` (coffescript must be installed first)
  - JavaScript: when `options.eval` is set to `true`
* Easy to add additional parsers! pull requests welcome!


## Usage

```js
var matter = require('gray-matter');
matter('---\ntitle: Front Matter\n---\nThis is content.');
```

Returns:

```js
{ 
  orig: '---\ntitle: Front Matter\n---\nThis is content.',
  data: { title: 'Front Matter' },
  content: '\nThis is content.' 
}
```

That's it! Just pass a string and gray-matter returns an object.


***


## API
{%= apidocs("index.js") %}


## Options

> All methods exposed on the API accept an options object passed as the last argument

## options.parser
Type: `Function`

Default: `undefined`

Pass a custom parser on the options. This is useful if you need to, for example, define custom schemas for [js-yaml].

**Example**

```js
matter(str, {
  parser: require('js-yaml').safeLoad
});
```

## options.eval
Type: `Boolean`

Default: `false`

Evaluate coffee-script, CSON or JavaScript in front-matter. If you aren't aware of the dangers, google is your friend. 

However, if you are aware and you only use front-matter on, say, blog posts for a static site... this feature can be pretty useful.


## options.lang
Type: `String`

Default: `yaml`

The parser to use on the extracted front matter. 

YAML is parsed by default, and the languages listed below are parsed automatically if the language is specified after the first delimiter (e.g. `---`). 

Valid languages are:

* `yaml`
* `json`
* `coffee` (to enable you must first install [coffe-script][])
* `cson` (to enable you must first install [coffe-script][])
* `toml` (to enable you must first install [toml][])
* `js`|`javascript`

**Example**

To parse coffee front matter, you would define it as follows:

```js
---coffee
title: 'coffee functions'
user: 'jonschlinkert'
fn:
  reverse = (src) ->
    src.split('').reverse().join('')
---

<%= description %>
<%= reverse(user) %>
```

## options.delims
Type: `String`

Default: `---`

Open and close delimiters can be passed in as an array of strings. 

**Example:**

```js
// format delims as a string
matter.read('file.md', {delims: '~~~'});
// or an array (open/close)
matter.read('file.md', {delims: ['~~~', '~~~']});
```

would parse:

<pre>
~~~
title: Home
~~~
This is the {{title}} page.
</pre>


## Example usage

Given we have a page, `abc.html`, containing:

```html
---
title: YAML Front matter
description: This is a page
---
<h1>{{title}}</h1>
```

then running the following in the command line:

```js
matter('abc.html');
```
returns

```json
{
  "data": {
    "title": "YAML Front matter",
    "description": "This is a page"
  },
  "content": "<h1>{{title}}</h1>",
  "original": "---\ntitle: YAML Front matter\n---\n<h1>{{title}}</h1>"
}
```

## Benchmarks

**Benchmarks for building the [bootstrap-blog][]**

gray-matter would process all markdown posts in the [bootstrap-blog][] **20 times** before the [front-matter][] library finished processing it once.

```
front-matter.js x 271 ops/sec ±2.68% (80 runs sampled)
gray-matter.js x 4,294 ops/sec ±0.86% (91 runs sampled)
```


**Misc**

gray-matter is 12-20x faster than [front-matter] when content or front matter actually exist.

```bash
{%= docs("benchmark/last.md") %}
```

## Why?

> Why another YAML Front Matter library?

Because other libraries we tried failed to meet our requirements with [Assemble][]. Some most of the libraries met most of the requirements, but _none had all of them_. Here are the most important:

* Be usable, if not simple
* Allow custom delimiters
* Use a dependable and well-supported library for parsing YAML and other languages
* Don't fail when no content exists
* Don't fail when no front matter exists
* Have no problem reading YAML files directly
* Have no problem with complex content, including **non-front-matter** fenced code blocks that contain examples of YAML front matter. Other parsers fail on this.
* Should return an object with three properties:
  - `data`: the parsed YAML front matter, as a JSON object
  - `content`: the contents as a string, without the front matter
  - `orig`: the "original" content


[bootstrap-blog]: https://github.com/twbs/bootstrap-blog/tree/gh-pages/_posts
